#
# librefe.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@cd.xdsl.ne.jp>
#
#   This program is free software.
#

class Refe

  class SearchError < StandardError; end

  INDEX_FILE = 'index.refe'

  def initialize( m, s )
    @itable = {}
    invert_table m, '#'
    invert_table s, '.'
    @m_table = m    # methods
    @s_table = s    # singleton methods
  end

  def invert_table( tab, type )
    it = @itable
    tab.each do |cname, methods|
      nm = "#{cname}#{type}"
      methods.each_key do |name|
        (it[name] ||= []).push nm
      end
    end
  end

  def lookup( klass, method )
    tab = (klass[-1,1] == '.') ? @s_table : @m_table
    m = tab[ klass[0..-2] ]  or notfound 'class', klass
    d = m[ method ]          or notfound 'method', method

    puts d.strip
  end

  def lookup_class( klass )
    tab = (klass[-1,1] == '.') ? @s_table : @m_table
    m = tab[klass]           or notfound 'class', klass

    puts "class #{klass[0..-1]}"
begin
    packed_print m.keys.sort
rescue TypeError
  p m.keys
  raise
end
  end

  def lookup_method( method )
    ent = @itable[ method ]  or notfound 'method', method
    packed_print ent.sort
  end

  def packed_print( arr )
    max = 60
    len = 0
    arr.each do |i|
      if len + 1 + i.size > max then
        puts; len = 0
      end
      print i, ' '; len += i.size
    end
    puts
  end

  def notfound( label, name )
    raise SearchError, "no such #{label}: #{name}"
  end

end
